/**************************************************************************
 *
 * bldargv --  Scan a string, setting pointers to the space-delimited
 *             words within it.  Null terminate each word except the
 *             last (which should already be null terminated, anyway).
 *             Strip leading spaces from the args (that is...point
 *             past them).
 *
 *   Usage:
 *        Pass this routine a pointer to an array which will hold
 *        the pointers to the arguments, a pointer to the string buffer,
 *        and the maximum number of pointers your array can hold. It
 *        will return the number of pointers generated.  Note that if
 *        there are more words in the buffer than you have pointer slots
 *        for, the last pointer will be to the remainder of the buffer.
 *
 *        Confused?  Check this out:
 *
 *             char string[] = "    c:\test.prg   arga   argb   argc"
 *             char *argv[3];
 *             numargs = bldargv(argv,string,3);
 *
 *        This sequence will yield the following:
 *
 *             numargs = 3
 *             argv[0] -> "c:\test.prg"
 *             argv[1] -> "arga"
 *             argv[2] -> "argb   argc"
 *
 *        Note that the leading spaces were removed from the arguments,
 *        but the imbedded spaces within the 'rest of the buffer' 3rd
 *        arg were left intact.
 *
 * Maintenance:
 *  04/13/87 - Original version. 
 *
 **************************************************************************/

bldargv(argv,p_strbuf,maxarg)
register char *argv[],*p_strbuf;
int  maxarg;
{
register int  argc;

argc = 0;

while(*p_strbuf)
     {
     while (*p_strbuf == ' ')      /* Cruise through spaces */
          p_strbuf++;
     if (*p_strbuf)                /* Found non-space, but skip if null. */
          {
          argv[argc++] = p_strbuf; /* Found start of word, mark its loc. */
          if (argc == maxarg)
               return(argc);       /* If out of pointer slots, return.   */
          while (*p_strbuf != ' ') /* Cruise through word (non-blank).   */
               {
               if (*p_strbuf == 0x00)   /* If end of string, return.     */
                    return(argc);
               p_strbuf++;              /* Else look at next char.       */
               }
          *p_strbuf++ = 0x00;      /* Found end of word, null-term word. */
          }
     }
return(argc); /* If string ended on a run of spaces we'll exit this way. */
}
