/*       This function simulates
     the get and put of GFA */



/*

	Requirements:
		work_out[] be declared as global.
		
		 get_screen_block(handle, x1, y1, x2, y2, loc)
 			 This function will take any part of the screen and get
			 it just as the GFA basic get command. loc is a char pointer
			 to the block of memory where the image is stored. This block
			 must be reserved previous to the call and be large enough to 
			 hold the gotten image. The image really starts at byte seven,
 			 the first six are used for control.

		 put_screen_block(handle, x1, y1, loc, mode)
			 This function will put a previously gottten block to the screen at
			 x1, y1. Mode is any of the 16 gem writing modes. The format of loc
 			 is exactly as GFA so any block gotten with a GFA get and loaded into
			 loc will be able to be used.

  		 get_screen(loc)
  		 	 This function will save the entire screen.

  		 put_screen(loc)
  		 	 This function will restore the entire screen.

		 word_width(pixels)
			 This function is used internally by the above two. It takes a pixels 
 			 count and determines how many words it takes to hold those pixels.
  
		 screen_block_size(x1, y1, x2, y2)
			 This functions returns the number of bytes required to hold a screen
			 rectangle of x1,y1   x2,y2. The number returned includes the six byte
			 overhead for control values.
 */


#include <structs.h>
#include <osbind.h>
#include <stdio.h>



static MFDB screen, memory;
extern work_out[];



void get_screen_block(handle, x1, y1, x2, y2, loc)

	int x1, y1,   /* x and y of screen */
 		x2, y2,
 		handle;   /* handle of application */ 
 		char *loc;  /* pointer to memory block */

{ 
			int x_dim, y_dim;
register	char *data;   /* pointer to data portion */ 
register	int  *data_int;
register	int pxy[8];   /* points array */
register	MFDB *memory_ptr = &memory;


	data = loc + 6;    /* the data starts 6 bytes in */
	data_int = (int *) loc;  /* change loc to a int for loading 6 bytes */
	x_dim = x2 - x1 + 1;   /* compensate for zero coord */
	y_dim = y2 - y1 + 1;
	
	vq_extnd(handle, 1, work_out);	/* get number of planes */
	data_int[2] = work_out[4];
	
	screen.f_addr = 0;   /* set up for screen */

	memory_ptr->f_addr = (long) data;
	memory_ptr->f_w = data_int[0] = x_dim - 1;
	memory_ptr->f_h = data_int[1] = y_dim - 1;
	memory_ptr->f_wdwidth = word_width(data_int[0]);
	memory_ptr->f_stand = 0;      /* Raster coords */
	memory_ptr->f_planes = data_int[2];
	memory_ptr->f_r1 = memory_ptr->f_r2 = memory_ptr->f_r3 = 0;

	pxy[0] = x1;
	pxy[1] = y1;
	pxy[2] = x2;
	pxy[3] = y2;

	pxy[4] = pxy[5] = 0;
	pxy[6] = x_dim - 1;
	pxy[7] = y_dim - 1;

	vro_cpyfm(handle, 3, pxy, &screen, &memory);

}

/********************************/

void put_screen_block(handle, x1, y1, loc, mode)

	int x1, y1,   /* x and y of screen */
		mode,   /* writing mode */
 		handle;   /* handle of application */
 		char *loc;  /* pointer to memory block */

{

	register	char *data;  	 /* pointer to data portion */
	register    int  *data_int;
	register	int pxy[8]; 	/* points array */
	register	MFDB *memory_ptr = &memory;

	data = loc + 6;    /* the data starts 6 bytes in */
	data_int = (int *) loc;  /* change loc to a int for loading 6 bytes */

	screen.f_addr = 0;   /* set up for screen */

	memory_ptr->f_addr = (long) data;
	memory_ptr->f_w = data_int[0];
	memory_ptr->f_h = data_int[1];
	memory_ptr->f_wdwidth = word_width(data_int[0]);
	memory_ptr->f_stand = 0; /* Raster coords */
	memory_ptr->f_planes = data_int[2];
	memory_ptr->f_r1 = memory_ptr->f_r2 = memory_ptr->f_r3 = 0;

	pxy[0] = pxy[1] = 0;
	pxy[2] = data_int[0];
	pxy[3] = data_int[1];

	pxy[4] = x1;
	pxy[5] = y1;
	pxy[6] = x1 + data_int[0];
	pxy[7] = y1 + data_int[1];

	vro_cpyfm(handle, mode, pxy, &memory, &screen);

}

/************************/
get_screen(dest)
char *dest;
{
char *source;
int len=32000;

	source = (char *)Physbase();

	bcopy(source, dest, len);
	
}


/*********************/
put_screen(source)
char *source;
{
char *dest;
int len=32000;

	dest = (char *)Physbase();

	bcopy(source, dest, len);
	
}

/***********************/
int word_width(pixels)
int pixels;
{

int result;

	result = pixels / 16;

	if ( result * 16 == pixels) {
		 return result;
		 }
	else {
		 return ++result;
		 }

}



/*****************************
   Determine screen block size 
******************************/

int screen_block_size(handle, x1, y1, x2, y2)

	int x1, y1,
		x2, y2,
		handle;
{ 
	int planes,
		dum1,dum2;
		
	vq_extnd(handle, 1, work_out);	/* get number of planes */
	planes = work_out[4];

	dum1 = (y2 - y1) + 1;
	dum2 =  (word_width(x2-x1)*2) * dum1 * planes;
	return dum2 + 6;
}

