#ifndef XREF_H
#define XREF_H

/* --------------------------------------------------------------------- */
/*
 * Include Files for Library Function Prototypes 
 */
#include  <stdio.h>
#include  <fcntl.h>
#include  <malloc.h>
#include  <time.h>
#include  <strings.h>
#include  <ctype.h>
#include  <osbind.h>
#include  <libb.h>

/* --------------------------------------------------------------------- */
/*
 * Some Valuable Definitions 
 */
#define      O_RAW  O_BINARY

#define      TEMPORARY_FILENAME   "CPR$.TMP"

#define      MAJOR_VERSION  4
#define      MINOR_VERSION  1

#define      MAX_S          256
#define      LENGTH         60
#define      WIDTH          132
#define      N_FILES        100
#define      TOC_LEN        1024
#define      MAX_FNAME_SIZE 50
#define      MAX_HDR_BYTES  500
#define      HEADER_SIZE    3

/* ------------------------------------------------------------------------- */
/*
 * These Macros are used to determine valid characters for Table of Contents
 */

/*
 * returns true is letter, digit, '_', '*'
 */
#define isnamechr(c)		(isalnum(c) || (c == '*') || (c == '_'))

/*
 * returns true if letter, digit, '_' , '*', or ' '
 */
#define isidchr(c)			(isnamechr(c) || (c == ' '))

/*
 * returns true if letter, digit, '_', '*', ' ', '\t' or '\n'
 */
#define isiechr(c)			(isidchr(c) || c == '\t' || c == '\n')

/* --------------------------------------------------------------- */

/*
 * Internal Program Function Prototypes 
 */
#ifdef ANSI_PROTO
#define ARGS(x)		(x)
#else
#define ARGS(x)		()
#endif


	void main ARGS( (int argc, char **argv) );
	void Usage ARGS( (void) );
	void StartTempFile ARGS( (void) );
	void EndTempFile ARGS( (void) );
	void DumpTempFile ARGS( (void) );
	int store_mod_call ARGS( (struct cptr * modlist[], int m1, int m2) );
	int store_call ARGS( (struct tnode * stree, int tindex) );
	int store_uses ARGS( (struct tnode * stree, int tindex, int p) );
	int search_call ARGS( (struct tnode * stree, int tindex, struct cptr ** cp) );
	int search_use ARGS( (struct tnode * stree, int tindex, struct cptr ** cp) );
	void module_map ARGS( (struct cptr * modlist[], char *title) );
	void proc_map ARGS( (int (*sroutine) (),  char *title) );
	void DumpCrossReference ARGS( (void) );
	void Done ARGS( (void) );
	void List ARGS( (void) );
	void NewPage ARGS( (void) );
	void NewFile ARGS( (void) );
	int PutLine ARGS( (char *l) );
	void NewFunction ARGS( (void) );
	void BreakPage ARGS( (void) );
	void PutHeader ARGS( (void) );
	int keycomp ARGS( (char *elem1, char *elem2) );
	void write_xref ARGS( (char *s) );
	int LooksLikeFunction ARGS( (char *s) );
	void AddToTableOfContents ARGS( (void) );
	void DumpTableOfContents ARGS( (void) );
	char *strupper ARGS( (char *str) );
	char *root_name ARGS( (char *ns) );
	void SortTableOfContents ARGS( (void) );
	char *EndComment ARGS( (char *p, int *flag) );
	char *EndString ARGS( (char *p, int *flag) );
	char *expand ARGS( (char *string) );
	char *Rs ARGS( (char *s) );
	int Cmemb ARGS( (char a, char *b) );
	int StarDot ARGS( (char *file_string) );
	void ReportMemoryError ARGS( (void) );
	void ReadHeaderFile ARGS( (char *header_file) );
	void ReadOutFile ARGS( (char *out_file) );
	int ReadResponseFile ARGS( (char *response_file) );
	int c_break ARGS( (void) );
	void file_stats ARGS( (char *filename) );
	int printer_code_init ARGS( (void) );
	void normal_print ARGS( (void) );
	void emphasized_print_on ARGS( (void) );
	void emphasized_print_off ARGS( (void) );
	void print_double_width ARGS( (void) );
	void compressed_print ARGS( (void) );
	void new_print_page ARGS( (void) );
	void underline_on ARGS( (void) );
	void underline_off ARGS( (void) );
	void pspace ARGS( (int ns) );
	char *strupper ARGS( (char *str) );
	char *strsave ARGS( (char *s) );
	void *strfree ARGS( (char *s) );
	char *strcompress ARGS( (char *s) );
	int CrossRefFunction ARGS( (char *s) );
	int searchtree_index ARGS( (struct tnode * stree, char *word, int tindex) );
	int searchtree_word ARGS( (struct tnode * stree, char *word, int *tindex) );
	struct tnode *entertree ARGS( (struct tnode * stree, char *word, int *lindex, int *tindex) );
	void clear_screen ARGS( (void) );
	long get_time_date ARGS( (char *s) );
	void parse_command_line ARGS( (char **argv, int argc) );
	void check_for_switch ARGS( (void) ); /* check */
	int  check_for_space_arg ARGS( (void) ); /* check */
	void walk_file_list ARGS( (void) );
	void disk_information ARGS( (disk_info_1 *space1, disk_info_2 *space2, int drive) );
	void PutLine1 ARGS( (char *string) ); 

/* -------------------------------------------------------------------- */

/*
 *    Some structure Definitions 
 */
struct esc_codes
{
    char *id;
    char *prt_init;
    char *prt_compress;
    char *prt_normal;
    char *prt_double_width_on;
    char *prt_double_width_off;
    char *prt_emphasize_on;
    char *prt_emphasize_off;
    char *prt_new_page;
    char *prt_underline_on;
    char *prt_underline_off;
};

#define  PRINTERS 3

typedef char ckey_type[10];

	/*
	 *  Some fields need values assigned on declaration
	 */
#ifdef GLOBAL
	char *STDIN_C = "\n";	/* special string pointer value to */

/* --------------------------------------------------------------------- */
/**
 * The following string is the definition of tab stops.  Every 'T' is a
 * tab, any other character is not.  It is currently  defined as a tab
 * every 3 spaces.  The comment below helps if you want to modify this,
 * each row being 0+n, 50+n, 100+n, and 150+n, where n is the number
 * above the declaration.  Don't try to number each row with a comment,
 * because you'll notice that the '\'s make it one big string.
 */

char *TabDef = "\
--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T\
--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T--T";

struct esc_codes print_code[PRINTERS] =
{
      {  "Epson FX",
	 "\x01B\x040",
	 "\x00F",
	 "\x012",
	 "\x00E",
	 "\x014",
	 "\x01B\x045",
	 "\x01b\x046",
	 "\x00C",
	 "\x01B\x02D\x000",
	 "\x01B\x02D\x001"
      },
      {  "DeskJet ",
	 "\x01B(s16.67h0B\x01b&a10L",
	 "\x0",
	 "\x01B(s16.67h0B",
	 "\x01b(s5h3B",
	 "\x01B(s16.67h0B",
	 "\x01b(s3B",
	 "\x01b(s0B",
	 "\x01B&l0H",
	 "\x01b&d2D",
	 "\x01b&d@"
      },
      {  "ASCII Text",
	 "",
	 "",
	 "",
	 "",
	 "",
	 "",
	 "",
	 "\f",
	 "",
	 ""
      },
};

struct esc_codes *printer;


ckey_type ckeys[] =
	{
 	"auto     ", 	"break    ", 	"case     ", 	"cdecl    ",
 	"char     ", 	"const    ", 	"continue ", 	"default  ",
 	"do       ", 	"double   ", 	"else     ", 	"enum     ",
 	"extern   ", 	"far      ", 	"float    ", 	"for      ",
 	"fortran  ", 	"goto     ", 	"huge     ", 	"if       ",
 	"int      ", 	"interrupt", 	"long     ", 	"near     ",
 	"pascal   ", 	"register ", 	"return   ", 	"short    ",
 	"signed   ", 	"sizeof   ", 	"static	 ", 	"struct   ",
 	"switch   ", 	"typedef  ", 	"union    ", 	"unsigned ",
 	"void     ", 	"volatile ", 	"while    "};

#else
	extern char *STDIN_C;			/* special string pointer value to */
	extern char *TabDef;			/* Tab definitions */
	extern struct esc_codes print_code[];	/* Table of printer escape codes */
	extern struct esc_codes *printer;		/* ptr to printer escape codes table */
	extern ckey_type ckeys[];
#endif

/* --------------------------------------------------------------------- */
/*
 * Global Variables 
 */
#ifdef GLOBAL
#define CLASS
#else
#define CLASS extern
#endif

/* --------------------------------------------------------------------- */

	CLASS FILE *TempFile;			/* Program Temporary File		*/
	CLASS FILE *File;				/* Current File being worked on */
	CLASS FILE *Ofile;				/* file output goes to			*/
	CLASS FILE *Rfile;				/* Response File handle			*/
	CLASS FILE *Hfile;				/* Header   File handle			*/
	
	CLASS disk_info_1 disk_space_1;		/* struct used for finding free disk space */
	CLASS disk_info_2 disk_space_2;		/* struct used for finding free disk space */
	
	CLASS char file_time_buff[20];	/* file time is placed here		*/
	CLASS char file_date_buff[20];	/* file date is placed here		*/
	
	CLASS int Braces;				/* Keeps track of brace depth	*/
	CLASS int LineNumber;			/* Count output lines			*/
	CLASS int PageNumber;			/* You figure this one out		*/
	CLASS int SectionNumber;		/* and this one to				*/
	CLASS int PageLength;			/* Normal paper length			*/
	CLASS int PageWidth;			/* normal page width			*/
	CLASS int OnePerPage;			/* non-zero puts 1 function/page*/
	CLASS int NumLines;				/* non-zero puts line # in file */
	CLASS int Number;
	CLASS int WantSorted;		/* Sort the table of contents	*/
	CLASS int PageEnd;			/* Accounts for space at bottom */
	CLASS int InComment;		/* Indicates if inside Comment  */
	CLASS int InXComment;		/* Indicates if inside Comment (Xref) */
	CLASS int InString;			/* Indicates if inside String	*/
	CLASS int InXString;		/* Indicates in String for Xref Function */
	CLASS int double_strike;	/* # of times + 1 to write filename             */
	CLASS int header_flag;		/* non-zero enables print codes/Dates/Page #'s  */
	CLASS int header_lines;		/* number of extra lines from header file       */
	CLASS int extra_lines;		/* non-zero enables extra lines after functions */
	CLASS int table_contents;	/* non-zero indicates Table of Contents Only    */
	CLASS int sectionflg;       /* non-zero indicates print by section          */

/*
 *  ASCII printer as default
 * zero = Epson, 1 = Deskjet, 2 = ASCII(none)
 */
	CLASS int nprinter;

	CLASS unsigned long memfree;	/* amount of local memory left for file buffer  */

	CLASS char *TempName;			/* pointer to temporary filename	*/
	CLASS char *fname[N_FILES];		/* file names to be worked on		*/

/*
 * flag a file as being stdin
 */
	CLASS char *Name;				/* Current file name				*/
	CLASS char *ProgName;			/* This programs Path and Name		*/

	CLASS char *Toc[TOC_LEN];		/* pointers to files in TOC			*/
	CLASS int TocPages[TOC_LEN];	/* page numbers						*/
	CLASS int TocIndex[TOC_LEN];
	CLASS int TocCount;				/* index into TOC arrays			*/
	CLASS char *cproc;
	CLASS int nextf;				/* index into fname[]				*/

	CLASS char header_buffer[MAX_HDR_BYTES];	/* 500 byte buffer for user defined Page Headers */
	CLASS char FunctionName[255];		/* Work area for parsing Func names */
	CLASS char Todayv[45];		/* Today's Time and Date	*/


#define  KEYSIZE  sizeof(ckeys) / sizeof(ckey_type)

/*------------------------------- tree types ------------------*/
	struct cptr
	{
    	struct cptr *next_ptr;
    	int proc;
	};
	
	struct entrytype
	{
    	char *proc;
    	int module;
    	unsigned int index;
    	struct cptr *calls;
    	struct cptr *uses;
	};
	
	struct tnode
	{
    	struct entrytype *this_entry;
    	struct tnode *left;
    	struct tnode *right;
	};

	CLASS struct tnode *proctree;
	CLASS struct cptr *modcall[N_FILES];
	CLASS struct cptr *moduse[N_FILES];

	CLASS int module_index,
		    proc_index,
    		last_module_index,
    		last_proc_index;

#define  MAX_XREF 6144
	struct call_type
	{
    	int module;
    	int proc;
    	char *call;
	};

	CLASS struct call_type call_list[MAX_XREF];
	CLASS int num_call;
#endif
