#define GLOBAL
#include "xref.h"

/*************************************************************************/
void main (argc, argv)
    int argc;
    char **argv;
{
	unsigned long int total_disk_free = 0;
	unsigned int ch;
	file_info *fblock;

	nextf = 0;
	double_strike = 1;		/* default to no double strike       */
	header_flag = 1;		/* default to codes and headers      */
	header_lines = 0;		/* no extra lines from header file   */
	extra_lines = 0;		/* no extra lines after functions    */
	PageNumber = 1;			/* Start with page 1 				 */
	SectionNumber = 1;		/* Start with section 1 			 */
	PageLength = LENGTH;	/* Init page length					 */
	PageWidth = WIDTH;		/* Init page width					 */
	OnePerPage = 0;			/* non-zero puts 1 function/page	 */
	NumLines = 0;			/* non-zero puts line # in file 	 */
	WantSorted = 1;			/* Sort the table of contents 		 */
	nprinter = 2;			/* ASCII printer as default			 */
    
	cproc = NULL;
	Number = 1;
	last_proc_index = last_module_index = 0;
	num_call = 0;
	TocCount = 0;
	proctree = NULL;
	Ofile = stdout;

	bzero(modcall, sizeof(modcall));
	bzero(moduse, sizeof(moduse));

	ProgName = *argv;		/* grab pointer to the programs name */

	get_time_date(Todayv);	/* grab the SYSTEM time and date  */

	if (argc == 1)			/* if only the program name given */
		Usage ();			/* if no args, show instructions */

	/* Print the Banner */
	fprintf (stderr, "\n\nCPR - A  C  Source Code Printing and Cross Reference Utility\nVersion 4.0  July 1990\n\n");

	parse_command_line(argv, argc);		/* parse the command line for switches
											and such */

	if (!nextf)		/* No files were specified */
		fname[nextf++] = STDIN_C;		/* so use STDIN as the input file */

	PageEnd = PageLength - (1 + PageLength / 20);

	StartTempFile ();					/* open temporary Output file */
	printer_code_init ();

	walk_file_list();					/* walk thru file list */

	if (PageNumber > 1 || LineNumber > 0)
		BreakPage ();

	EndTempFile ();						/* Close Temporary File */

	fprintf (stderr, "\n");

	/* get bytes free on disk */
	disk_information(&disk_space_1, &disk_space_2, 0);

	fblock = get_file_info(TEMPORARY_FILENAME);	/* get info on temp file */

	/* see if copy can be output */ 
	if (fblock->fsize >= disk_space_2.total_bytes_free) {
		fprintf (stderr, "Insufficient Disk Space!\n\
						\nMinimum of %lu bytes needed.\
						\n%lu bytes are available\n\
						\nDo You wish to continue? (Y/N)",
						fblock->fsize, disk_space_2.total_bytes_free);

		do {
			ch = toupper (getchar ());
			} while ((ch != 'Y') && (ch != 'N'));

		if (ch == 'N')
			Done ();				/* Delete Temporary File */

		fprintf (stderr, "\n\n");
		} /* end of if */

	/* if TOC and file enabled or TOC only enabled */
	if ((!table_contents) || (table_contents == 1))
		DumpTableOfContents ();				/* Print Table of Contents */

	DumpCrossReference ();					/* Print cross reference */

	/* if TOC and file enabled or no TOC  */
	if ((!table_contents) || (table_contents == -1))
		DumpTempFile ();					/* then dump the file */

	Done ();								/* Clean things up and leave */

}  /* end of main () */


/**********************************************************************
 *   Walk through the file list, processing every file
 **********************************************************************/
void walk_file_list()
{
	register int i;

	for (i = 0; i < nextf; i++) {
		if (fname[i] == STDIN_C) {			/* if no files */
			File = stdin;					/* assign stdin as input */
			Name = "Standard Input";		/* assign name for printout */
			fprintf (stderr, "Accepting Input from STDIN, ^Z to END\n");
			}  /* end of if */
		else {	/* there was a filename */
			if ((File = fopen (Name = fname[i], "r")) == NULL) {
				fprintf (stderr, "%s: Can't open file \"%s\"\n", 
							ProgName, Name);
				continue;				/* keep going if not found */
				} /* end of if */
			else {						/* file opened OK so read */
				fprintf (stderr, "\nReading File #%3d of %3d: \" %s \"\n",
								i + 1, nextf, fname[i]);

				}  /* end of else */
			} /* end of else */

		if (cproc)
			free (cproc);

		cproc = NULL;
		module_index = i;

		List ();						/* Read File in and process data */

		if (File != stdin) {			/* if it's not STDIN */
			fclose (File);				/* close file & deallocate buffer */

			fprintf (stderr, "\t\t\t\t\tTable Entries: %4d  Cross Refs: %4d",
							TocCount, num_call);
			} /* end of if */
		} /* end of for loop */
} /* end of walk_file_list() */
