/*
 *     All the code for parsing the switches is here
 */

#include "xref.h"

static	char	**argv;			/* my copy of argv for this program */
static	int		argc;			/* my copy of argc */
static	char	**first_parm;	/* a saved pointer to the first argv */


void parse_command_line(v, c)
	char	**v;			/* argv */
	int		c;				/* argc */
{
	int	loop_control;


	argv = v;				/* move argv from main into my argv */
	first_parm = argv++;	/* save pointer to program name & point to next */
	argc = c;				/* move argc from main into my argc */

	loop_control = TRUE;	/* Loop until FALSE */

	while (loop_control) {

		if (**argv == '-')		/* check for switch flag "-" */
			check_for_switch();
		else {
			/* Argument wasn't a switch parmeter */
	    	if (nextf >= N_FILES) {
				fprintf (stderr, "%s: too many files\n", ProgName);
				exit (1);
			    } /* end of if */

		    fname[nextf++] = *argv; 	/* copy pointer to filename */
			} /* end of else */

		if ( ++(argv) == (first_parm) + argc)	/* see if another argument */
			loop_control = FALSE;				/* no more args */
	    } /* end of for loop */
} /* end of parse_command_line */


/**********************************************************************/
void check_for_switch()
{
	int	number;

	switch (toupper (*(*argv+1))) /* and go thru the switches */
		{
	    case '@':
			/* Response file */
			nextf = ReadResponseFile (*argv);
			break;
			
		case 'A':
			/*  "*.???" Print all specified files */
			nextf = StarDot (*argv);
			break;

		case 'C':
			/* Print codes/Dates/Time and page #  */
			header_flag = 0;
			break;

		case 'D':
			/* Double Strike function names */
			double_strike = 3;
			break;

		case 'H':
			/* Header file */
			ReadHeaderFile (*argv);
			break;

		case 'F':
			/* Header file */
			ReadOutFile (*argv);
			break;

		case 'L':
			/* Number of lines per page */
			PageLength = check_for_space_arg();
			break;

		case 'N':
			/* Number Source Lines */	
			NumLines = 1;
			break;

		case 'O':
			/* 1 function per page */
			OnePerPage = 1;
			break;

		case 'P':
			/* Starting Page Number */
			PageNumber = check_for_space_arg();
			break;

		case 'Q':
			/* Print by Section */
			SectionNumber = check_for_space_arg(); 
			sectionflg = 1;
			break;

		case 'S':
			/* Sorted Table of Contents */
			WantSorted = 0;
			break;

		case 'T':
			/* Print Table of Contents Only */
			table_contents = 1;		/* enable flag */

			if (*(*argv+2) == '-')		/* if disabling TOC */
				table_contents = -1;	/* disable it */
			break;

		case 'W':
			/* Page Width */
			PageWidth = number = check_for_space_arg();
			break;

		case 'X':
			/* Extra Lines after Functions */
			extra_lines = 1;
			break;

		case '\0':
			/* End of string?(out of enviroment) */
			if (nextf >= N_FILES) {
				fprintf (stderr, "%s: too many files\n", ProgName);
				exit (1);
				} /* end of if */

			fname[nextf++] = STDIN;	/* any in excess goes to STDIN */
			break;

		default:
			/* No arguments given */
			Usage ();				/* show program instructions */
			break;

		} /* end of switch */
}


/*
 *  In many instances a number may follow a switch. In case the user
 *  entered a space between the switch and the number (-X 66) we need
 *  to check next argument. If a space follows the switch and the 
 *  next argument is not a number, then print the help and exit.
 */
int check_for_space_arg()
{
	if (!(*(*argv+2))) {	/* if -X ?? (ie.. a space) */
		if ( ++(argv) == (first_parm) + argc)	/* see if another argument */
			Usage ();			/* if no then print Help */

		if (!isdigit (**argv))	/* is it a number? */
			Usage ();			/* if no then print Help */

		return(atoi(*argv));		/* return the number */
		} /* end of if */
	else  /* Switch came as -X?? form (with no space) */
		{
		if (!isdigit (*(*argv+2)))	/* is it a number? */
			Usage ();				/* if no then print Help */

		return(atoi(*argv+2));		/* return the number */
		} /* end of else */
}

/*****************************************************************************/
void ReadOutFile (outfile)
    char *outfile;
{
    outfile += 2;    /* strip the "-F" from the string */

    if ((Ofile = fopen (outfile, "w")) == NULL) {
		fprintf (stderr, "Can't Open Out File %s\n", outfile);
		exit (1);	/* terminate abruptly */
		}
}
